<?php
namespace Controllers;

// Контроллер работы с API Агорты
class Api extends Controller {

	public function __construct() {
		parent::__construct();
		$api_data=$this->controller->get_api_config();
		$this->model=new \Models\Api($this->f3, $api_data['domen'], $api_data['pass'], $api_data['store_id'], $api_data['count']);
	}

	// Установка СНО
	function sno() {
		$res['success']=true;
		$txt='';
		$this->db->exec('UPDATE synchronization SET time_start=?', time());
		$sno=$this->model->sno();
		if($sno['success']===true) {
			$r=$this->db->exec("UPDATE fr_settings SET sno=?", $sno['sno']);
			if($r>=0) {
				$txt=$this->messages->message('save_successful')['txt'];
			}
		}
		else { // если ответ от сервера не успешный
			$txt='';
			foreach ($sno['error'] as $er) {
				$txt.=$this->messages->message($er)['txt'].'<br>';
			}
			if(!$sno['debug'])$sno['debug']=array();
			$res['debug']=$sno['debug'];
			$res['success']=false;
			if($sno['success']===false) {
				$this->logs->save_log($txt.implode(';<br>', $res['debug']));
			}
		}

		$res['txt']=$txt;
		$this->db->exec('UPDATE synchronization SET time_start=0');

		echo json_encode($res);
		exit();
	}

	// Функция синхронизации пользователей
	function users() {
		$res['success']=true;
		if($this->f3->get('GET.reload')) {
			$res['reload']='.users_list';
			$res['close_all_window']=true;
		}
		$txt='';
		$this->db->exec('UPDATE synchronization SET time_start=?', time());
		$users=$this->model->users(); // получаем пользователей
		if($users['success']===true) { // если все получили
			// Добавляем группы
			$this->db->exec('DELETE FROM usergroups');
			$g=new \Models\Usergroups($this->db, $this->f3);
			foreach ($users['usergroups'] as $group) {
				$group_res=$g->save_group($group);
				$mess=$this->messages->message($group_res['txt']);
				if(!$group_res['success']) { // Если ошибка, то добавляем к тексту сообщения
					$res['success']=$group_res['success'];
					$txt.='<span class="false">'.$group['title'].' - '.$mess['txt'].'</span><br>';
				}
			}

			// Добавляем пользователей
			$this->db->exec('DELETE FROM users');
			$this->db->exec('VACUUM');
			foreach ($users['users'] as $user) {
				$user_res=$this->users->save_user($user);
				$mess=$this->messages->message($user_res['txt']);
				if(!$user_res['success']) {  // Если ошибка, то добавляем к тексту сообщения
					$res['success']=$user_res['success'];
					$txt.='<span class="false">'.$user['name'].' - '.$mess['txt'].'</span><br>';
				}
			}

			if($txt=='') { // если текст сообщения еще не сформирован, то добавляем из последнего запроса
				$txt=$this->messages->message('success_users_synchronization')['txt'];
			}
			else {
				$this->logs->save_log($txt);
			}
		}
		else { // если ответ от сервера не успешный
			$txt='';
			foreach ($users['error'] as $er) {
				$txt.=$this->messages->message($er)['txt'].'<br>';
			}
			if(!$users['debug'])$users['debug']=array();
			$res['debug']=$users['debug'];
			$res['success']=false;
			if($users['success']===false) {
				$this->logs->save_log($txt.implode(';<br>', $res['debug']));
			}
		}

		$this->db->exec('UPDATE synchronization SET time_start=0, users_update=?', time());
		$res['txt']=$txt;
		$res['logs_n']=$this->f3->get('logs_n');
		echo json_encode($res);
		exit();
	}

	// Загрузка категорий и товаров
	function products_load($time_update=0) {
		$res['success']=true;
		$txt='';

		$data=$this->model->products($time_update); // загружаем продукты
		if($data['success']===true) {
			// Добавляем категории товаров
			$sql=array();
			$val=array();
			foreach ($data['categories'] as $cat) {
				$sql[]="INSERT OR REPLACE INTO productcategories (id, parent_id, title, date_delete, date_update) VALUES (?, ?, ?, ?, ?)";
				$val[]=array(
					$cat['id'],
					$cat['parent_id'],
					$cat['title'],
					$cat['date_delete'],
					$cat['date_update']
				);
				$time_update=$cat['date_update'];
			}
			if(!empty($sql)) {
				if(!$this->db->exec($sql, $val)) {
					$txt.='<br>'.$this->messages->message('db_error')['txt'];
				}
				sleep(1);
			}

			// Добавляем товары и штрих-коды
			$sql=array();
			$val=array();
			$bsql=array();
			$bval=array();
			$dsql=array();
			$dval=array();
			foreach ($data['products'] as $prod) {
				$sql[]="INSERT OR REPLACE INTO products (id, code, ppr, fast, title, short_title, img, category_id, stock, price, vat, vat_type, date_delete, date_update) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
				$val[]=array(
					$prod['id'],
					$prod['code'],
					$prod['ppr'],
					$prod['fast'],
					$prod['title'],
					$prod['short_title'],
					$prod['img'],
					$prod['category_id'],
					$prod['stock'],
					$prod['price'],
					$prod['vat'],
					$prod['vat_type'],
					$prod['date_delete'],
					$prod['date_update']
				);
				$dsql[]="DELETE FROM `barcodes` WHERE product_id=?";
				$dval[]=array($prod['id']);
				$time_update=$prod['date_update'];
				$barcodes=explode(',', $prod['barcodes']);
				if(!empty($barcodes)) {
					foreach ($barcodes as $barcode) {
						$bsql[]="INSERT OR REPLACE INTO barcodes (product_id, barcode) VALUES (?, ?)";
						$bval[]=array(
							$prod['id'],
							$barcode
						);
					}
				}

			}
			if(!empty($sql)) {
				if(!$this->db->exec($sql, $val)) {
					$txt.='<br>'.$this->messages->message('db_error')['txt'];
				}
				sleep(1);
			}
			if(!empty($dsql)) {
				$d=$this->db->exec($dsql, $dval);
				if($d===false) {
					$txt.='<br>'.$this->messages->message('db_error')['txt'];
				}
			}
			if(!empty($bsql)) {
				if(!$this->db->exec($bsql, $bval)) {
					$txt.='<br>'.$this->messages->message('db_error')['txt'];
				}
			}
			if($txt!='') {
				$this->logs->save_log($txt);
			}

			// Если массивы с продуктами и категориями были не пустые, то надо продолжить
			if(count($data['categories'])>0 || count($data['products'])>0) {
				$time=time();
				$this->db->exec('UPDATE synchronization SET time_start=?, time_update=?', array($time, $time_update));
				$txt.='<br>'.$this->messages->message('update')['txt'];
				$res['next_step']=$time;
			}
			else {
				$txt='<br>'.$this->messages->message('save_successful')['txt'];
				$this->db->exec('UPDATE synchronization SET time_start=?, time_update=?', array(0, $time_update));
			}

		}
		else { // Если произошла ошибка
			foreach ($data['error'] as $er) {
				$txt.=$this->messages->message($er)['txt'].'<br>';
			}
			$res['txt']=$txt;
			if(!$data['debug'])$data['debug']=array();
			$res['debug']=$data['debug'];
			$res['success']=false;
			if($data['success']===false) {
				$this->logs->save_log($txt.implode(';<br>', $res['debug']));
			}
			$this->db->exec('UPDATE synchronization SET time_start=?, time_update=?', array(0, $time_update));
		}
		$res['txt']=$txt;

		return $res;
	}

	// Загрузка обновленныйх категорий и товаров
	function products() {
		// Проверяем не запущена ли уже синхронизация.
		$sinch=$this->db->exec("SELECT * FROM synchronization WHERE time_start=0 OR time_start<".(time()-600)." LIMIT 1");
		if(!$sinch) {
			$res=array(
				'success'=>false,
				'txt'=>$this->messages->message('sync_is_already_running')['txt']
			);
		}
		else {
			$this->db->exec('UPDATE synchronization SET time_start=?', array(time()));
			$time_update=$sinch[0]['time_update'];
			$res=$this->products_load($time_update);
			if($this->f3->get('auto_sinch') && $res['success']) {
				$res['txt']=$this->messages->message('success_products_synchronization')['txt'];
			}
			$this->db->exec('UPDATE synchronization SET time_start=0');
		}
		$res['logs_n']=$this->f3->get('logs_n');
		echo json_encode($res);
		exit();
	}

	// Полная загрузка категорий и товаров
	function products_all() {
		$time_start=$this->f3->get('GET.time_start'); // Узнаем время запуска синхронизации
		if(!$time_start) $time_start=0; // если время не задано, то это начало синхронизации
		// Проверяем не запущена ли уже синхронизация.
		$sinch=$this->db->exec("SELECT * FROM synchronization LIMIT 1");
		if($sinch[0]['time_start']==$time_start || ($time_start==0 && $sinch[0]['time_start']<time()-600)) {
			if($time_start>0) { // Если синхронизация не только что началась, то время обновления берем из таблицы
				$time_update=$sinch[0]['time_update'];
			}
			else { // если синхронизация только что началась, то время обновления 0, чтобы получить все товары
				$time_update=0;
			}
			if($time_start==0) { // Если синхронизация только началась, то очищаем таблицы
				$this->db->exec('DELETE FROM productcategories');
				$this->db->exec('DELETE FROM products');
				$this->db->exec('DELETE FROM barcodes');
				$this->db->exec('VACUUM');
			}
			$this->db->exec('UPDATE synchronization SET time_start=?', array(time()));
			$res=$this->products_load($time_update);
		}
		else {
			$mess=$this->messages->message('sync_is_already_running');
			$res['txt']=$mess['txt'];
			$res['success']=false;
		}

		if($res['next_step']) {
			$res['next_step']='/api/products_all/?time_start='.$res['next_step'];
		}
		echo json_encode($res);
		exit();
	}

	// Клиенты
	// Загрузка клиентов
	function partners_load($time_update=0) {
		$res['success']=true;
		$txt='';

		$data=$this->model->partners($time_update); // загружаем клиентов
		if($data['success']===true) {
			// Добавляем клиентов
			$sql=array();
			$val=array();
			foreach ($data['partners'] as $p) {
				$sql[]="INSERT OR REPLACE INTO partners (card, partner_id, name, email, phone, discount, date_delete) VALUES (?, ?, ?, ?, ?, ?, ?)";
				$val[]=array(
					$p['card'],
					$p['partner_id'],
					$p['name'],
					$p['email'],
					$p['phone'],
					$p['discount'],
					$p['date_delete']
				);
				$time_update=$p['date_update'];
			}
			if(!empty($sql)) {
				if(!$this->db->exec($sql, $val)) {
					$txt.='<br>'.$this->messages->message('db_error')['txt'];
				}
				sleep(1);
			}
			if($txt!='') {
				$this->logs->save_log($txt);
			}

			// Если массив с клиентами был не пустый, то надо продолжить
			if(count($data['partners'])>0) {
				$time=time();
				$this->db->exec('UPDATE synchronization SET time_start=?, partners_update=?', array($time, $time_update));
				$txt.='<br>'.$this->messages->message('update')['txt'];
				$res['next_step']=$time;
			}
			else {
				$txt='<br>'.$this->messages->message('save_successful')['txt'];
				$this->db->exec('UPDATE synchronization SET time_start=?, partners_update=?', array(0, $time_update));
			}
		}
		else { // Если произошла ошибка
			foreach ($data['error'] as $er) {
				$txt.=$this->messages->message($er)['txt'].'<br>';
			}
			if(!$data['debug'])$data['debug']=array();
			$res['debug']=$data['debug'];
			$res['success']=false;
			if($data['success']===false) {
				$this->logs->save_log($txt.implode(';<br>', $res['debug']));
			}
			$this->db->exec('UPDATE synchronization SET time_start=?, partners_update=?', array(0, $time_update));
		}
		$res['txt']=$txt;
		return $res;
	}

	// Загрузка обновленныйх клиентов
	function partners() {
		// Проверяем не запущена ли уже синхронизация.
		$sinch=$this->db->exec("SELECT * FROM synchronization WHERE time_start=0 OR partners_update<".(time()-600)." LIMIT 1");
		if(!$sinch) {
			$res=array(
				'success'=>false,
				'txt'=>$this->messages->message('sync_is_already_running')['txt']
			);
		}
		else {
			$this->db->exec('UPDATE synchronization SET time_start=?', array(time()));
			$time_update=$sinch[0]['partners_update'];
			if(!$time_update)$time_update=0;
			$res=$this->partners_load($time_update);
			if($this->f3->get('auto_sinch') && $res['success']) {
				$res['txt']=$this->messages->message('success_partners_synchronization')['txt'];
			}
			$this->db->exec('UPDATE synchronization SET time_start=0');
		}
		$res['logs_n']=$this->f3->get('logs_n');
		echo json_encode($res);
		exit();
	}

	// Полная загрузка клиентов
	function partners_all() {
		$time_start=$this->f3->get('GET.time_start'); // Узнаем время запуска синхронизации
		if(!$time_start) $time_start=0; // если время не задано, то это начало синхронизации
		// Проверяем не запущена ли уже синхронизация.
		$sinch=$this->db->exec("SELECT * FROM synchronization LIMIT 1");
		if($sinch[0]['time_start']==$time_start || ($time_start==0 && $sinch[0]['time_start']<time()-600)) {
			if($time_start>0) { // Если синхронизация не только что началась, то время обновления берем из таблицы
				$time_update=$sinch[0]['partners_update'];
			}
			else { // если синхронизация только что началась, то время обновления 0, чтобы получить все товары
				$time_update=0;
			}
			if($time_start==0) { // Если синхронизация только началась, то очищаем таблицы
				$this->db->exec('DELETE FROM partners');
				$this->db->exec('VACUUM');
			}
			$this->db->exec('UPDATE synchronization SET time_start=?', array(time()));
			$res=$this->partners_load($time_update);
		}
		else {
			$mess=$this->messages->message('sync_is_already_running');
			$res['txt']=$mess['txt'];
			$res['success']=false;
		}

		if($res['next_step']) {
			$res['next_step']='/api/partners_all?time_start='.$res['next_step'];
		}
		echo json_encode($res);
		exit();
	}
	//

	// Отдаем чеки
	function tickets($json=true) {
		$sinch=$this->db->exec("SELECT * FROM synchronization LIMIT 1");
		if($sinch[0]['time_start']<time()-600) {
			$this->db->exec("UPDATE synchronization SET time_start=?", time());
			$time_update=$sinch[0]['tickets_update'];
			if(!$time_update)$time_update=0;
			$receipts=new \Models\Receipts($this->db, $this->f3);
			$tickets=$receipts->select_receipts(array('r.time>? AND type>0', $time_update), array('order'=>'r.time'));
			if($tickets) {
				$res=$this->model->tickets($tickets);
				if(!empty($tickets)) {
					$n=count($tickets)-1;
					$tickets_update=$tickets[$n]['time'];
				}
				else {
					$tickets_update=$time_update;
				}
				if($res['success']===true) {
					$res['txt']=$this->messages->message('save_successful')['txt'];
					$this->db->exec("UPDATE synchronization SET time_start=0, tickets_update=?", $tickets_update);
				}
				else {
					foreach ($res['error'] as $er) {
						$txt.=$this->messages->message($er)['txt'].'<br>';
					}

					$res['txt']=$txt;
					if($res['success']===false) {
						if(!$res['debug'])$res['debug']=array();
						$this->logs->save_log($txt.implode(';<br>', $res['debug']));
					}
					$res['success']=false;
					/*if($res['noupdate']) {
						$this->db->exec("UPDATE synchronization SET time_start=0");
					}
					else {
						$this->db->exec("UPDATE synchronization SET time_start=0, tickets_update=?", $tickets_update);
					}*/
					$this->db->exec("UPDATE synchronization SET time_start=0");
				}
			}
			else {
				$res=array(
					'success'=>true,
					'txt'=>$this->messages->message('save_successful')['txt']
				);
				$this->db->exec("UPDATE synchronization SET time_start=0, tickets_update=?", $time_update);
			}
			if($this->f3->get('auto_sinch') && $res['success']) {
				$res['txt']=$this->messages->message('success_tickets_synchronization')['txt'];
			}
		}
		else {
			$mess=$this->messages->message('sync_is_already_running');
			$res['txt']=$mess['txt'];
			$res['success']=false;
		}
		if($json) {
			$res['logs_n']=$this->f3->get('logs_n');
			echo json_encode($res);
			exit();
		}
	}

	// Загрузка префиксов для весового штрих-кода и штрих-кода карты клиента
	function barcodes_pr() {
		$res['success']=true;
		$txt='';
		$this->db->exec('UPDATE synchronization SET time_start=?', time());
		$pr=$this->model->barcodes_pr();
		if($pr['success']===true) {
			$sql=array();
			$val=array();
			foreach ($pr['barcodes_pr'] as $name => $value) {
				$sql[]="INSERT OR REPLACE INTO settings (name, value) VALUES (?, ?)";
				$val[]=array($name, $value);
			}
			$r=$this->db->exec($sql, $val);
			if($r!==false) {
				$txt=$this->messages->message('save_successful')['txt'];
			}
			else {
				$txt=$this->messages->message('db_error')['txt'];
			}
		}
		else { // если ответ от сервера не успешный
			$txt='';
			foreach ($pr['error'] as $er) {
				$txt.=$this->messages->message($er)['txt'].'<br>';
			}
			if(!$pr['debug'])$pr['debug']=array();
			$res['debug']=$pr['debug'];
			$res['success']=false;
			if($pr['success']===false) {
				$this->logs->save_log($txt.implode(';<br>', $res['debug']));
			}
		}

		$res['txt']=$txt;
		$this->db->exec('UPDATE synchronization SET time_start=0');

		echo json_encode($res);
		exit();
	}

	// Авто синхронизация
	function auto() {
		$logs=$this->logs->select_log(array('read=?', 0));
		$res['logs_n']=count($logs);
		$this->f3->set('logs_n', $res['logs_n']);
		$sinch=$this->db->exec("SELECT * FROM synchronization WHERE time_start=0 OR time_start<".(time()-600)." LIMIT 1");
		if(!$sinch) {
			$res['success']=false;
			$res['txt']=$this->messages->message('sync_is_already_running')['txt'];
		}
		elseif($sinch[0]['auto_sinch']<0) {
			return;
		}
		elseif($sinch[0]['auto_sinch']>time()-$this->f3->get('synchronization_timer')*60) {
			$res['success']=false;
			$res['txt']=$this->messages->message('no_synchronization_required')['txt'];
		}
		else {
			$this->f3->set('auto_sinch', 1);
			if($sinch[0]['users_update']<time()-3600) {
				$this->db->exec("UPDATE synchronization SET auto_sinch=?", time());
				$this->users();
			}
			elseif($sinch[0]['users_update']<time() && $sinch[0]['users_update']>time()-120) {
				$this->db->exec("UPDATE synchronization SET auto_sinch=?", array(time()));
				$this->partners();
			}
			elseif($sinch[0]['method']==1) {
				$this->db->exec("UPDATE synchronization SET auto_sinch=?, method=?", array(time(), 0));
				$this->tickets();
			}
			$this->db->exec("UPDATE synchronization SET auto_sinch=?, method=?", array(time(), 1));
			$this->products();
		}
		echo json_encode($res);
		exit();
	}
}
